--[[
	CS_Utils.lua - Is a support module file for the CombatStats
	World of Warcraft addon.  This module has utility functions for CombatStats.
	
	Copyright (C) 2011 Brian Beardmore - wowaddon(at)realisticsoftware(dot)com

	Revisions:
		2010.11.04 BAB	WoW 4.0.1 fixes
		2010.01.22 BAB	added fixLvlDropMenuButtons function
		2009       BAB  GetGlobal call improvement
		2008.10.03 BAB	Added isTrueValue function
		2007.09.19 BAB	Added isTextNumbers function
		2007.09.17 BAB	Fix the numToBool & boolToNum function names.
		2007.08.26 BAB	The file was created from functions extracted from the
						prior CombatStats.lua file.  Functions were written by DmgInc,
						other authors and Brian Beardmore.
						
	License:
		This program is free software; you can redistribute it and/or
		modify it under the terms of the GNU General Public License
		as published by the Free Software Foundation; either version 2
		of the License, or (at your option) any later version.

		This program is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
		GNU General Public License for more details.

		You should have received a copy of the GNU General Public License
		along with this program(see GPL.txt); if not, write to the Free Software
		Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
]]

-- Debug switch - set to true, to enable debug output for this module
local debug = false;


-------------------------------------------------------------------------------
-- Function Imports
-------------------------------------------------------------------------------


-------------------------------------------------------------------------------
-- Function Prototypes
-------------------------------------------------------------------------------
local load;

local clockTextFormatPart;
local clockFormatDeltaTime;

local clearLocalInfo;
local resetLocalInfo;
local appendMenuItem;
local appendMenuChkItem;
local appendMenuItemSeparator;
local appendMenuItemSubmenu;
local appendMenuItemSubmenuItem;
local fixLvlDropMenuButtons;

local numToBool;
local boolToNum;
local isTrueValue;

local isTextNumbers;

local debugPrint;
local msgPrint;

-------------------------------------------------------------------------------
-- Data Members
-------------------------------------------------------------------------------

local _L = {
	info = nil;
};

-------------------------------------------------------------------------------
-- Constants
-------------------------------------------------------------------------------

local INT_GENERAL_DURATION_MILLISEC = CLOCK_TIME_MILLISECOND			--	improve by creating this dynamically from INT_GENERAL_DURATION_SEC
local INT_GENERAL_DURATION_MILLISEC_P1 = CLOCK_TIME_MILLISECONDS

-------------------------------------------------------------------------------
-- Uses
-------------------------------------------------------------------------------
--[[ 	INT_GENERAL_DURATION_SEC, INT_GENERAL_DURATION_SEC_P1,
		INT_GENERAL_DURATION_DAYS, INT_GENERAL_DURATION_DAYS_P1,
		INT_GENERAL_DURATION_HOURS, INT_GENERAL_DURATION_HOURS_P1,
		INT_GENERAL_DURATION_MIN, INT_GENERAL_DURATION_MIN_P1,
		INT_GENERAL_DURATION_SEC, INT_GENERAL_DURATION_SEC_P1
]]

-------------------------------------------------------------------------------
-- Loads this module.
-------------------------------------------------------------------------------
function load()
	clearLocalInfo();
end

--=============================================================================
-- date time text format functions
--=============================================================================

-------------------------------------------------------------------------------
-- clockTextFormatPart
-------------------------------------------------------------------------------
function clockTextFormatPart(fmt, val, fmtS)
	local part;
	
	if( ( val ~= 1 ) and ( fmtS ~= nil ) )
	then
		part = format(fmtS, val);
	else
		part = format(fmt, val);
	end
	
	return part;
end

-------------------------------------------------------------------------------
-- clockFormatDeltaTime
-------------------------------------------------------------------------------
function clockFormatDeltaTime(datetime, maxparts, include_ms)
	-- see Blizzards GlobalStrings.lua file for the definition of INT_GENERAL_DURATION_xxxxxxx;
	local d, h, m, s, ms;
	local text = "";
	local parts = 0;
	
	maxparts = maxparts or 2;
	include_ms = include_ms or false;
	
	-- there is strangeness when the datetime input is negative;
	if ( datetime == 0 ) then return clockTextFormatPart(INT_GENERAL_DURATION_SEC, 0, INT_GENERAL_DURATION_SEC_P1); end
	d, h, m, s = ChatFrame_TimeBreakDown(datetime);
	ms = math.floor(((datetime % 1) * 1000)+0.5);
	if( d > 0 ) then
		text = text..clockTextFormatPart(INT_GENERAL_DURATION_DAYS, d, INT_GENERAL_DURATION_DAYS_P1);
		parts = parts + 1;
	end
	if( ((parts > 0) and (parts < maxparts)) or ((parts == 0) and (h > 0)) ) then
		if (parts > 0) then
			text = text..", ";
		end
		text = text..clockTextFormatPart(INT_GENERAL_DURATION_HOURS, h, INT_GENERAL_DURATION_HOURS_P1);
		parts = parts + 1;
	end
	if( ((parts > 0) and (parts < maxparts)) or ((parts == 0) and (m > 0)) ) then
		if (parts > 0) then
			text = text..", ";
		end
		text = text..clockTextFormatPart(INT_GENERAL_DURATION_MIN, m, INT_GENERAL_DURATION_MIN_P1);
		parts = parts + 1;
	end
	if( ((parts > 0) and (parts < maxparts)) or ((parts == 0) and (s > 0)) ) then
		if (parts > 0) then
			text = text..", ";
		end
		text = text..clockTextFormatPart(INT_GENERAL_DURATION_SEC, s, INT_GENERAL_DURATION_SEC_P1);
		parts = parts + 1;
	end
	if (include_ms) then
		if( ((parts > 0) and (parts < maxparts)) or ((parts == 0) and (ms > 0)) ) then
			if (parts > 0) then
				text = text..", ";
			end
			text = text..clockTextFormatPart(INT_GENERAL_DURATION_MILLISEC, ms, INT_GENERAL_DURATION_MILLISEC_P1);
			parts = parts + 1;
		end
	end
	
	return text;
end

--=============================================================================
-- drop down menu functions
--=============================================================================

function clearLocalInfo()
	_L.info = UIDropDownMenu_CreateInfo();
end

function resetLocalInfo()
	-- resets the unusually info fields only
	_L.info.checked = nil;
	_L.info.disable = nil;
	_L.info.notClickable = nil;
	_L.info.hasArrow = nil;
end

function appendMenuItem(theText, theValue, theFunc, theOwner)
	resetLocalInfo();
	_L.info.text = theText;
	_L.info.value = theValue;
	_L.info.func = theFunc;
	_L.info.owner = theOwner;
	UIDropDownMenu_AddButton(_L.info, UIDROPDOWNMENU_MENU_LEVEL);
end

function appendMenuChkItem(theText, theValue, theFunc, theOwner)
	resetLocalInfo();
	_L.info.text = theText;
	_L.info.value = theValue;
	_L.info.func = theFunc;
	_L.info.owner = theOwner;
	_L.info.checked = 1;
	UIDropDownMenu_AddButton(_L.info, UIDROPDOWNMENU_MENU_LEVEL);
end

function appendMenuItemSeparator(theText, theOwner)
-- for some unknown reason this function requires its own local variable to not mess up the entire menu
	--local info;
	--info = {};
	--info = UIDropDownMenu_CreateInfo();
	
	resetLocalInfo();
	_L.info.text = theText;
	_L.info.value = nil;
	_L.info.func = nil;
	_L.info.disable = 1;
	--_L.info.notClickable = 1;		-- this is the problem
	_L.info.owner = theOwner;
	UIDropDownMenu_AddButton(_L.info, UIDROPDOWNMENU_MENU_LEVEL);
end

function appendMenuItemSubmenu(theText, theValue, theFunc, theOwner)
	resetLocalInfo();
	_L.info.text = theText;
	_L.info.value = theValue;
	_L.info.func = theFunc;
	_L.info.owner = theOwner;
	_L.info.hasArrow = 1;
	UIDropDownMenu_AddButton(_L.info, UIDROPDOWNMENU_MENU_LEVEL);
end

function appendMenuItemSubmenuItem(theText, theValue, theFunc, theOwner)
	resetLocalInfo();
	_L.info.text = theText;
	_L.info.value = theValue;
	_L.info.func = theFunc;
	_L.info.owner = theOwner;
	UIDropDownMenu_AddButton(_L.info, UIDROPDOWNMENU_MENU_LEVEL);
end

function fixLvlDropMenuButtons()
	-- workaround for 3.3 menu bug
	local index
	local button
	local listFrame = _G["DropDownList"..UIDROPDOWNMENU_MENU_LEVEL]
	if (listFrame == nil) then return end
	local listFrameName = listFrame:GetName()
	if ((listFrameName == nil) or (listFrame.numButtons == nil)) then return end
	local correctButtonLvl = (listFrame:GetFrameLevel()+1)
	for index=1, listFrame.numButtons, 1 do
		button = _G[listFrameName.."Button"..index]
		button:SetFrameLevel(correctButtonLvl)
	end
end

--=============================================================================
-- variable manipulator functions
--=============================================================================

function numToBool(theNum)
	if (theNum == nil) then return nil end
	local theNumType = type(theNum)
	
	if (theNumType == "string") then
		theNum = tonumber(theNum)
	elseif (theNumType ~= "number") then
	end

	return (theNum ~= 0)
end

function boolToNum(theBool)
	if (theBool == nil) then return nil end
	if (type(theBool) ~= "boolean") then return nil end
	
	if (theBool) then
		return 1
	end
	return 0
end

function isTrueValue(theValue)
	if (theValue == nil) then return nil end
	
	if (type(theValue) == "boolean") then
		return theValue
	elseif (type(theValue) == "number") then
		return (theValue ~= 0)
	elseif (type(theValue) == "string") then
		local lowerValue = string.lower(theValue)
		if ( (lowerValue == "yes") or
			(lowerValue == "on") or
			(lowerValue == "true") ) then return true
		elseif ( (lowerValue == "no") or
			(lowerValue == "off") or
			(lowerValue == "false") ) then return false
		else
			return ( tonumber(theValue) ~= 0 )
		end
	end
	
	return nil
end

--=============================================================================
-- Miscellaneous functions
--=============================================================================

function isTextNumbers(theText)
	local aNum = tonumber(theText);
	
	if (aNum == nil) then
		return false;
	end
	
	--debugPrint("chk: "..theText..", num: "..aNum);
	-- this check does not work "theText == TEXT(aNum)"
	return (string.len(theText) == string.len(aNum));
end

--=============================================================================
-- Print functions
--=============================================================================

function debugPrint(msg, r, g, b, frame, id)
	if (not debug) then return; end
	
	local info = ChatTypeInfo["SYSTEM"]
	r = r or 1.0		-- info.r
	g = g or 1.0		-- info.g
	b = b or 1.0		-- info.b
	id = id or info.id
	frame = frame or DEFAULT_CHAT_FRAME
	
	frame:AddMessage(msg,r,g,b,id)
end

function msgPrint(msg, r, g, b, frame, id)
	local info = ChatTypeInfo["SYSTEM"]
	r = r or 1.0		-- info.r
	g = g or 1.0		-- info.g
	b = b or 1.0		-- info.b
	id = id or info.id
	frame = frame or DEFAULT_CHAT_FRAME
	
	frame:AddMessage(msg,r,g,b,id)
end

--=============================================================================
-- Initialization
--=============================================================================
if (CombatStats and CombatStats.Utils) then return end;
debugPrint("CS_Utils.lua loaded");

-------------------------------------------------------------------------------
-- Public API
-------------------------------------------------------------------------------
if (not CombatStats) then CombatStats = {}; end

CombatStats.Utils = {
	Load = load;
	
	ClockFormatDeltaTime = clockFormatDeltaTime;

	AppendMenuItem = appendMenuItem;
	AppendMenuChkItem = appendMenuChkItem;
	AppendMenuItemSeparator = appendMenuItemSeparator;
	AppendMenuItemSubmenu = appendMenuItemSubmenu;
	AppendMenuItemSubmenuItem = appendMenuItemSubmenuItem;
	FixLvlDropMenuButtons = fixLvlDropMenuButtons;
	
	NumToBool = numToBool;
	BoolToNum = boolToNum;
	IsTrueValue = isTrueValue;
	
	IsTextNumbers = isTextNumbers;
	
	DebugPrint = debugPrint;
	MsgPrint = msgPrint;
}

-------------------------------------------------------------------------------
-- Initialize before using
-------------------------------------------------------------------------------
load();
